<?php
/*
File: gbsign.inc.php
Project: Dasc's PHP Guestbook
Author(s): Daniel Schlyder <dsc@c2i.net>
Last modified: 2002-02-06

Copyright (c) 2001, 2002 Daniel Schlyder. All rights reserved.
(See README.txt)
*/

// Routines to print HTML form and add new entries
// ===============================================

require_once("inc/gbconfig.inc.php");
require_once("inc/gbcommon.inc.php");

// helps echoMenu() in 'gbview.inc.php' decide whether form is displayed
// above or not and consequently whether an anchor name (#entriesStart)
// should be defined for the top menu and pointed to by all entry page links
define("FORM_DISPLAYED", true);



echo "<h1>Sign guestbook</h1>\n\n";

// Determine action to take when script is called
if (isset($HTTP_POST_VARS['sign']))
{
	echo ($result = addEntry());
	if (strstr($result, "Error"))
		echoForm(true);
}
else
	echo echoForm(false);



// print HTML form
function echoForm($fillForm)
{
	global
		$cfg,
		$HTTP_GET_VARS;

	if ($fillForm)
		global $HTTP_POST_VARS;
	else
		echo "<p>Please fill in the <strong>bold</strong> fields; the rest "
			. "are optional.</p>\n\n";

	echo "<form action=\"" . getenv("SCRIPT_NAME");
	if (isset($HTTP_GET_VARS['start']))
		echo "?start=" . $HTTP_GET_VARS['start'];
	echo "\" method=\"post\">\n";
	
	echo "<table>\n";

	echoInputTag("Name", "name", $HTTP_POST_VARS['name'], 30, true);
	echoInputTag("E-mail", "email", $HTTP_POST_VARS['email'], 30);

	echo "<tr>\n";
	echo "\t<td></td>\n";
	echo "\t<td>\n";
	echo "\t\t(<input type=\"checkbox\" name=\"hideEmail\"";
	if (!$fillForm || isset($HTTP_POST_VARS['hideEmail']))
		echo " checked=\"checked\"";
	echo " /> e-mail address visible only to guestbook's owner)\n";
	echo "\t</td>\n";
	echo "</tr>\n";

	echoInputTag("Homepage", "homepage", $HTTP_POST_VARS['homepage'], 50);
	echoInputTag("City", "city", $HTTP_POST_VARS['city'], 30);
	echoInputTag("Country", "country", $HTTP_POST_VARS['country'], 30);

	echo "<tr>\n";
	echo "\t<td class=\"label\" style=\"vertical-align: top\">\n";
	echo "\t\t<strong>Message:</strong>\n";
	echo "\t</td>\n";
	echo "\t<td>\n";
	echo "\t\t<textarea name=\"message\" rows=\"6\" cols=\"45\">";
	if (isset($HTTP_POST_VARS['message']))
		echo $HTTP_POST_VARS['message'];
	echo "</textarea>\n";
	echo "\t</td>\n";
	echo "</tr>\n";

	echo "<tr>\n";
	echo "\t<td></td>\n";
	echo "\t<td class=\"tagList\">\n";
	echo "\t\tAllowed tags: ";
	if ($cfg['allowableTags'] != "")
		echo htmlspecialchars(str_replace("><", ">, <",
			$cfg['allowableTags']));
	else
		echo "none";
	echo "\n\t</td>\n";
	echo "</tr>\n";
	
	echo "<tr>\n";
	echo "\t<td></td>\n";
	echo "\t<td>\n";
	echo "\t\t<input type=\"submit\" name=\"sign\" value=\"Submit\" />\n";
	echo "\t\t<input type=\"submit\" value=\"Reset\" />\n";
	echo "\t</td>\n";
	echo "</tr>\n";
	echo "</table>\n";
	echo "</form>\n";
}



// print HTML input tag
function echoInputTag($label, $varName, &$varValue, $size,
	$required = false)
{
	echo "<tr>\n";
	echo "\t<td class=\"label\">\n";
	
	if ($required)
		echo "\t\t<strong>" . $label . ":</strong>\n";
	else
		echo "\t\t" . $label . ":\n";
	
	echo "\t</td>\n";
	echo "\t<td>\n";
	
	echo "\t\t<input type=\"text\" name=\"" . $varName . "\" value=\"";
	if (isset($varValue))
		echo $varValue;
	echo "\" size=\"" . $size . "\" maxlength=\"" . $size * 2 . "\" />\n";

	echo "\t</td>\n";
	echo "</tr>\n";
}



// process submitted form, add entry to datafile, and send optional e-mail
// to guestbook's owner
function addEntry()
{
	global
		$cfg,
		$HTTP_POST_VARS;

	// remove whitespace, slashes, and HTML tags
	while (list($key, $value) = each($HTTP_POST_VARS))
	{
		if (ini_get("magic_quotes_gpc"))
			$HTTP_POST_VARS[$key] = strip_tags(stripSlashes(trim($value)),
				$cfg['allowableTags']);
		else
			$HTTP_POST_VARS[$key] = strip_tags(trim($value),
				$cfg['allowableTags']);
	}
	
	// check that required fields aren't empty
	if ($HTTP_POST_VARS['name'] == "")
		return error("<var>Name</var> field is empty.");
	if ($HTTP_POST_VARS['message'] == "")
		return error("<var>Message</var> field is empty.");

	// check that message field isn't too long
	if ($cfg['maxMsgLength'])
	{
		$msgLength = strlen($HTTP_POST_VARS['message']);
		if ($msgLength > $cfg['maxMsgLength'])
			return error("<var>Message</var> is too long ("
				. $msgLength . ").\nPlease use no more than "
				. $cfg['maxMsgLength'] . " characters.");
	}

	// check if e-mail address is valid
	if ($HTTP_POST_VARS['email'] != ""
		&& !validMailAddr($HTTP_POST_VARS['email']))
	{
		if ($cfg['ignoreAddrErrors'])
			$HTTP_POST_VARS['email'] = "";
		else
			return error("<var>" . $HTTP_POST_VARS['email']
				. "</var> is not a valid e-mail address.");
	}

	// remove "http://" from homepage address
	$HTTP_POST_VARS['homepage'] = str_replace("http://", "",
		$HTTP_POST_VARS['homepage']);

	// check if homepage address is valid
	if ($HTTP_POST_VARS['homepage'] != ""
		&& !validHomepageAddr($HTTP_POST_VARS['homepage']))
	{
		if ($cfg['ignoreAddrErrors'])
			$HTTP_POST_VARS['homepage'] = "";
		else
			return error("<var>" . $HTTP_POST_VARS['homepage']
				. "</var> is not a valid URL.");
	}
	
	// get host name
	$host = gethostbyaddr(getenv("REMOTE_ADDR"));
	
	// get time and date added (in UTC)
	$date = gmdate("Y-m-d H:i:s", time());
	
	// prepare entry for being appended to datafile
	$entry = $date . "|" . $host . "|"
		. prepareVar($HTTP_POST_VARS['name']) . "|"
		. prepareVar($HTTP_POST_VARS['email']) . "|"
		. $HTTP_POST_VARS['hideEmail'] . "|"
		. prepareVar($HTTP_POST_VARS['homepage']) . "|"
		. prepareVar($HTTP_POST_VARS['city']) . "|"
		. prepareVar($HTTP_POST_VARS['country']) . "|"
		. prepareVar($HTTP_POST_VARS['message'], true) . "\n";
	
	@ $fp = fopen($cfg['datafile'], "a");
	
	if (!$fp)
		return error("Failed to open datafile. Please try again.", true);
	
	if (!flock($fp, LOCK_EX))
		return error("Failed to lock datafile. Please try again.", true);

	fwrite($fp, $entry);
	flock($fp, LOCK_UN);
	fclose($fp);
	
	// email new entry to guestbook's owner
	if ($cfg['entryTo'] != "")
	{
		$subject = "New Entry: " . $HTTP_POST_VARS['name'];
		
		$content = "On " . substr($date, 0, 10)
			. " at " . substr($date, 11) . " (UTC), "
			. $HTTP_POST_VARS['name'] . " (" . $host
			. ") wrote the following in your guestbook:\n\n"
			. $HTTP_POST_VARS['message'] . "\n";

		if ($HTTP_POST_VARS['city'] != "" || $HTTP_POST_VARS['country'] != ""
			|| $HTTP_POST_VARS['homepage'] != "")
		{
			$content .= "\n--\n";
			if ($HTTP_POST_VARS['homepage'] != "")
				$content .= "Homepage: http://" . $HTTP_POST_VARS['homepage']
					. "\n";
			if ($HTTP_POST_VARS['city'] != ""
				|| $HTTP_POST_VARS['country'] != "")
			{
				$content .= "From: ";
				if ($HTTP_POST_VARS['city'] != "")
				{
					$content .= $HTTP_POST_VARS['city'];
					if ($HTTP_POST_VARS['country'] != "")
						$content .= ", " . $HTTP_POST_VARS['country'];
				}
				else
					$content .= $HTTP_POST_VARS['country'];
				$content .= "\n";
			}
		}
		
		$extraHeaders = "From: " . $cfg['mailFrom'];
		
		// add Reply-To field if entry contains an e-mail address
		if ($HTTP_POST_VARS['email'] != "")
			$extraHeaders .= "\nReply-To: " . $HTTP_POST_VARS['email'];
		
		@ mail($cfg['entryTo'], $subject, $content, $extraHeaders);
	}
	
	return msg("Guestbook signed. Thank you!");
}



// prepare variable for being written to datafile
function prepareVar($var, $allowNewline = false)
{
	// replace vertical bars with equivalent HTML entity and remove carriage
	// returns
	$var = str_replace(chr(13), "", str_replace("|", "&#124;", $var));
	
	if ($allowNewline)
		// replace linefeeds with HTML breaks
		return str_replace(chr(10), "<br />", $var);
	else
		// remove linefeeds
		return str_replace(chr(10), "", $var);
}



// e-mail address validation
function validMailAddr($addr)
{
	// split address into user and domain name, and check that they aren't
	// empty
	$addr = explode("@", $addr);
	if (sizeof($addr) != 2 || $addr[0] == "")
		return false;

	return validURL($addr[1]);
}



// homepage address validation
function validHomepageAddr($addr)
{
	// strip directories
	if ($pos = strpos($addr, "/"))
		$addr = substr($addr, 0, $pos);
	
	return validURL($addr);
}



// check if URL is valid
function validURL($addr)
{
	$addr = strtolower($addr);
	
	// split into domain levels and check that there's at least two
	$levels = explode(".", $addr);
	$sLevels = sizeof($levels);
	if ($sLevels < 2)
		return false;
	
	// get top level domain and check that it's either 2-3 characters long,
	// or one of the few 4-5 character names
	$tld = $levels[$sLevels - 1];
	if (strlen($tld) < 2 || (strlen($tld) > 3
		&& !ereg("arpa|info|name|aero|coop|museum", $tld)))
		return false;
	
	return true;
}

?>
