<?php
/*
File: gbview.inc.php
Project: Dasc's PHP Guestbook
Author(s): Daniel Schlyder <dsc@c2i.net>
Last modified: 2002-02-06

Copyright (c) 2001, 2002 Daniel Schlyder. All rights reserved.
(See README.txt)
*/

// Routines to display entries
// ===========================

require_once("inc/gbconfig.inc.php");
require_once("inc/gbcommon.inc.php");

echo echoEntries();



// display a page of entries
function echoEntries()
{
	global
		$cfg,
		$HTTP_GET_VARS; // for $start
	
	// load datafile into array
	@ $entries = file($cfg['datafile']);
	if ($entries === false)
	{
		if (file_exists($cfg['datafile']))
			return error("Failed to open datafile. Please try again.", true);
		else
			return error("Datafile not found.", true);
	}
	else if (empty($entries))
		return msg("Guestbook is empty.");

	$sEntries = sizeof($entries);

	// determine first entry on last page
	$tmp = $sEntries % $cfg['entriesPerPage'];
	if ($tmp)
		$startLastPage = $sEntries - $tmp;
	else
		$startLastPage = $sEntries - $cfg['entriesPerPage'];

	// display page with newest or oldest entries (depending on value of
	// $cfg['orderNewToOld']) if $HTTP_GET_VARS['start'] is not set or out of
	// range
	if (!isset($HTTP_GET_VARS['start'])
		|| $HTTP_GET_VARS['start'] >= $sEntries
		|| $HTTP_GET_VARS['start'] < 0)
	{
		if ($cfg['orderNewToOld'])
			$HTTP_GET_VARS['start'] = $startLastPage;
		else
			$HTTP_GET_VARS['start'] = 0;
	}

	// get entries to display
	$entries = array_slice($entries, $HTTP_GET_VARS['start'],
		$cfg['entriesPerPage']);
	
	if ($cfg['orderNewToOld'])
		// reverse array to display newest entries on top
		$entries = array_reverse($entries);

	if ($sEntries > $cfg['entriesPerPage'])
		echoMenu($HTTP_GET_VARS['start'], $startLastPage, true);

	echo "<h1>Guestbook entries " . ($HTTP_GET_VARS['start'] + 1) . " - "
		. min($sEntries, $HTTP_GET_VARS['start'] + $cfg['entriesPerPage'])
		. "</h1>\n\n";
	
	$sEntriesToDisplay = sizeof($entries);
	
	while (list($id, $val) = each($entries))
	{
		list($date, $host, $name, $email, $hideEmail, $homepage, $city,
			$country, $message) = explode("|", $val);

		echo "<h2>";
		
		if ($cfg['displayEntryNum'])
		{
			if ($cfg['orderNewToOld'])
				echo ($HTTP_GET_VARS['start'] + $sEntriesToDisplay - $id) . '. ';
			else
				echo ($HTTP_GET_VARS['start'] + $sEntriesToDisplay
					- ($sEntriesToDisplay - $id) + 1) . '. ';
		}
		
		echo $name;
		
		if ($city != "")
		{
			echo " - " . $city;
			if ($country != "")
				echo ", " . $country;
		}
		else if ($country != "")
			echo " - " . $country;

		echo "</h2>\n";

		echo "<p class=\"datePageMail\">" . $date . " (UTC)";
		if ($homepage != "")
			echo "\n| <a href=\"http://" . $homepage . "\">homepage</a>";
		if ($email != "" && !$hideEmail)
			echo "\n| <a href=\"mailto:" . $email . "\">e-mail</a>";
		echo "</p>\n";

		echo "<p>";
		if ($cfg['ignoreEnter'])
			echo str_replace("<br />", "\n", chop($message));
		else
			echo str_replace("<br />", "<br />\n", chop($message));
		echo "</p>\n\n";

		if ($id < $sEntriesToDisplay - 1)
		{
			echo "<p>";
			if ($cfg['hrImage'] != "")
				echo "<img src=\"" . $cfg['hrImage'] . "\" alt=\"\" />";
			else
				echo "<hr />";
			echo "</p>\n\n";
		}
	}

	if ($sEntries > $cfg['entriesPerPage'])
		echoMenu($HTTP_GET_VARS['start'], $startLastPage, false);
}



// print page menu
function echoMenu($start, $startLastPage, $isTopMenu)
{
	global $cfg;
	$pageName = getenv("SCRIPT_NAME");
	
	if (defined("FORM_DISPLAYED"))
		$aID = "#entriesStart";
	else
		$aID = "";
	
	if ($isTopMenu)
	{
		echo "<p class=\"navbarTop\">\n";
		if (defined("FORM_DISPLAYED"))
			echo "<a id=\"entriesStart\"></a>\n";
	}
	else
		echo "<p class=\"navbarBottom\">\n";
	
	if ($start > 0)
		echo "[<a href=\"" . $pageName . "?start=0" . $aID
			. "\">First</a>]\n";
	
	if ($start - $cfg['entriesPerPage'] > 0)
		echo "[<a href=\"" . $pageName . "?start="
			. ($start - $cfg['entriesPerPage']) . $aID . "\">Previous</a>]\n";

	echo "<strong>Page " . ($start / $cfg['entriesPerPage'] + 1) . " of "
		 . ($startLastPage / $cfg['entriesPerPage'] + 1) . "</strong>\n";

	if ($start + $cfg['entriesPerPage'] < $startLastPage)
		echo "[<a href=\"" . $pageName . "?start="
			. ($start + $cfg['entriesPerPage']) . $aID . "\">Next</a>]\n";

	if ($start != $startLastPage)
		echo "[<a href=\"" . $pageName . "?start=" . $startLastPage . $aID
			. "\">Last</a>]\n";

	echo "</p>\n\n";
}

?>
